import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

export interface JSONSourceAnnotationAttributes {
  key: string;
}

export class JSONSourceAnnotation extends Annotation {
  public readonly attributes: JSONSourceAnnotationAttributes = {
    key: undefined,
  };

  public constructor(key: string | JSONSourceAnnotationAttributes) {
    super();
    if (undefined === key) {
      throw new Error(`key attribute must be specified.`);
    }

    if (TypeUtil.isString(key)) {
      this.attributes.key = <string>key;
    } else {
      this.copyAttributes(this.attributes, <JSONSourceAnnotationAttributes>key);
    }
  }

  public propertyDecorator(target: Object, propertyKey: PropertyKeyType): void {
    console.log('JSONSource');
  }

  public parameterDecorator(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('JSONSource');
  }
}

export const JSONSource = Decorator.create(JSONSourceAnnotation);
